// Configuration service
'use strict';

const path = require('path');
const chalk = require('chalk');
const _ = require('lodash');
const winston = require('winston');
const cluster = require('cluster');

// Config Holders
var configs = [];

// Print Once
function _printOnce(callback){
  if((cluster.isMaster || (cluster.worker && cluster.worker.id==1)) && callback && process.env.NODE_ENV!='test'){
    callback();
  }
}

// Init/load config
function init(moduleName){
  if(!moduleName || moduleName==''){
    moduleName = 'server';
  }

  // Special RAW access to methods
  if(moduleName=='_raw'){
    return {
      init: init,
      get: get,
      load: load
    };
  }

  // Preload requested config
  get(moduleName);

  return {
    get: get,
    load: load
  };
}

// Load config file
function load(moduleName){
  if(!moduleName || moduleName==''){
    moduleName = 'server';
  }

  // Detect env
  var env = process.env.NODE_ENV || 'development';
  if(moduleName == 'server'){
    _printOnce(()=>{
      winston.info(chalk.blue(moduleName), 'environment:', chalk.green(env));
    });
  }

  // Load default config
  var defaultConfig;
  var defaultConfigFilePath;
  if(moduleName == 'server'){
    defaultConfigFilePath = path.join(global.mio.appPath, 'etc','default.json');
  }else{
    defaultConfigFilePath = path.join(global.mio.appPath, 'modules', moduleName, 'etc', 'default.json');
  }
  try {
    defaultConfig = require(defaultConfigFilePath);
  } catch (err) {
    if(moduleName == 'server' && process.env.NODE_ENV!='test'){
      winston.error(chalk.red('No default config file', defaultConfigFilePath));
    }
  }

  // Load config
  var config;
  var configFilePath;
  if(moduleName == 'server'){
    configFilePath = path.join(global.mio.appPath, 'etc', env+'.json');
  }else{
    configFilePath = path.join(global.mio.appPath, 'modules', moduleName, 'etc', env+'.json');
  }
  try {
    config = require(configFilePath);
    _printOnce(()=>{
      var mode = config.mode || env;
      winston.info('Using Configuration for', chalk.blue(moduleName), '-', chalk.blue(mode));
    });
  } catch (err) {
    if (err.code && err.code === 'MODULE_NOT_FOUND') {
      if(moduleName == 'server'){
        if(process.env.NODE_ENV!='test'){
          winston.error(chalk.red('No config file', configFilePath));
        }
        throw new Error('No config file');
      }
    } else {
      throw err;
    }
  }

  // Merge Env Config and Default config
  config = _.merge(defaultConfig, config);
  configs[moduleName] = config;

  // Set Winston config level according to config
  if(config && config.logs && config.logs.level){
    _printOnce(()=>{
      winston.level = config.logs.level;
      winston.info('Set winstone level:', chalk.green(config.logs.level));
    });
  }

  // Return loaded config
  return configs[moduleName];
}

// Get loaded config file
function get(moduleName){
  if(!moduleName || moduleName==''){
    moduleName = 'server';
  }

  if(configs[moduleName]){
    return configs[moduleName];
  }else{
    return load(moduleName);
  }
}

module.exports = init;
